#Uncomment next two lines if R packages are already installed
#install.packages("phyloseq")
#install.packages("igraph")
#install.packages("markovchain")
library(phyloseq)
library(igraph)
library(markovchain)

#setwd('<path_to_working_directory>')
rm(list = ls())

#set function to filter samples
samdat.prune_prev <- function(samdat) {
  GAP_MIN <- 120
  GAP_MAX <- 1000
  samdf <- data.frame(samdat)
  subjects <- unique(samdf$SubjectID)
  csub <- split(samdf, samdf$SubjectID)
  for(sub in subjects) {
    cc <- csub[[sub]]
    cc <- cc[order(cc$GDColl),]
    cc$PrevID <- c(NA, cc$SampleID[-nrow(cc)])
    del <- cc$GDColl - c(-999, cc$GDColl[-nrow(cc)])
    keep <- del>=GAP_MIN & del<=GAP_MAX
    if(sum(keep) == 0) {
      csub[[sub]] <- NULL
    } else {
      cc <- cc[keep,]
      csub[[sub]] <- cc
    }
  }
  return(do.call(rbind, csub))
}

##calculate transition probability
samdf = read.table("MarkovList.txt",header = T,sep = '\t',row.names = 1)
rownames(samdf) -> samdf$SampleID
samdf$type = as.factor(samdf$type)
CSTs <- levels(samdf$type)
nstates <- nlevels(samdf$type)
samdf_prev <- samdat.prune_prev(samdf)
rownames(samdf_prev) <- samdf_prev$SampleID
samdf_prev$PrevCST <- data.frame(samdf)[samdf_prev$PrevID,"type"]
samdf_prev$CurCST <- samdf_prev$type
ttab <- table(samdf_prev$PrevCST, samdf_prev$CurCST) # prevstate=row, curstate=col
trans <- matrix(ttab, nrow=nstates)
trans <- trans/rowSums(trans)  # Normalize row sums to 1
CSTtrans <- trans
colnames(CSTtrans) <- CSTs
rownames(CSTtrans) <- CSTs
t_persist <- -1/log(diag(CSTtrans))

##plot markov chain
mcPreg <- new("markovchain", states=CSTs,
              transitionMatrix = trans, name="PregCST")
netMC <- markovchain:::.getNet(mcPreg, round = TRUE)
wts <- E(netMC)$weight/100
edgel <- get.edgelist(netMC)
elcat <- paste(edgel[,1], edgel[,2])
elrev <- paste(edgel[,2], edgel[,1])
edge.curved <- sapply(elcat, function(x) x %in% elrev)
default.par <- par(no.readonly = TRUE)
plotMC <- function(object, ...) {
  netMC <- markovchain:::.getNet(object, round = TRUE)
  plot.igraph(x = netMC, ...)  
}
vert.sz <- 2*sapply(states(mcPreg), 
                        function(x) nrow(unique(sample_data(samdf)[sample_data(samdf)$type==x,"SubjectID"])))
vert.sz <- log(vert.sz)*5
vert.font.clrs <- c("white", "white", "white", "white")
edge.loop.angle = c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.14, 0, 0, 0, 0, 0)-0.45
layout <- matrix(c(0.6,0.95, 0.43,1, 0.3,0.66, 0.55,0.3), nrow=4, ncol=2, byrow=T)
#layout(matrix(c(1,1,2,2), 2, 2, byrow = TRUE), heights=c(1,10))
par(mar=c(0,1,1,1)+0.1)
edge.arrow.size=1
edge.arrow.width=1
edge.width = (6*wts + 0.1)
edge.labels <- as.character(E(netMC)$weight/100)
edge.labels[edge.labels<0.2] <- NA  # labels only for self-loops

##plot
plotMC(mcPreg, edge.arrow.size=edge.arrow.size, edge.arrow.width = edge.arrow.width,
       edge.label = edge.labels, edge.label.font=2, edge.label.cex=1.3, edge.label.color="black",
       edge.width=edge.width, edge.curved=edge.curved, 
       vertex.size=(vert.sz),
       vertex.label.font = 2, vertex.label.cex = 2,
       vertex.label.color = vert.font.clrs, vertex.frame.color = NA, 
       vertex.color = c("#CDBE6A","#89CFBE","#86A4CF","#DFAD9F"),
       layout = layout, edge.loop.angle = edge.loop.angle)

